@@ -1,9 +1,9 @@
 #include "Zmaxdef.h"
 c          common variables used in tracking ptcls.
        integer ToInteract, ToBeObserved, Truncated, Dead,
-     *         BorderL,  BorderH
+     *         BorderL,  BorderH, AngleLimit
        parameter(ToInteract = 1, ToBeObserved = 2, Truncated = 3,
-     *  BorderL = 4, BorderH =5,  Dead = 6)
+     *  BorderL = 4, BorderH =5,  Dead = 6, AngleLimit = 7)
 
        integer MaxInte
        parameter(MaxInte = 6)  !  Max number of kinds of interactions a particle can
@@ -59,6 +59,10 @@ c                                     (in 'xyz')
                               !  must be made t, when Epics treats muon.
                               !  if Cosmos uses Epics, this must be made to
                               !  be t/f depending on Epics mode, or Cosmos mode
+#if LABELING > 0
+        integer Labelcounter   ! label counter to put a lalel on each patcl.
+#endif
+
 	parameter (
 #ifdef MAX_PTCL
      *     MaxPtcl = MAX_PTCL
@@ -79,7 +83,12 @@ c
      *  ObserveAS, Nproduced, 
      *  MoveStat,  NumberOfInte, ProcessNo, 
      *  TargetNucleonNo, TargetProtonNo, Upgoing, UseTbl,
-     *  FromEpics 
+     *  FromEpics
+#if LABELING > 0
+     * , Labelcounter
+#endif
+
+ 
 
 
 
