@@ -13,7 +13,8 @@ c
 c         get endpoint coord. assuming no scat, no mag. effect. at first
 c         MovedTrack has complete information at path end.
        MovedTrack = TrackBefMove   !   copy TrackBefMove into MovedTrack. 
-       call cmoveStreight(IntInfArray(ProcessNo).length)
+       call cmoveStreight(IntInfArray(ProcessNo).length, 
+     *             TrackBefMove.vec.w)
 c         If a chargde ptcl, compute energy loss by dE/dx  and  reset energy
 c         in MovedTrack. If it is too large, path is truncated. Also compute
 c         deflection.
@@ -29,7 +30,7 @@ c          endif
        endif
        end
 c     --------------------------------------------------------
-       subroutine cmoveStreight(leng)
+       subroutine cmoveStreight(leng, dir)
 c        move a track by leng m
 c         after this is called,  MovedTrack has inf for moved pos.
 c         MovedTrack.pos.where may not be correct. It must be
@@ -44,15 +45,19 @@ c         observation layer.
 
 
        real*8  leng, cnewcos
+       record /coord/ dir        ! input.
 c
 c            new coord in xyz system.
 
        MovedTrack.pos.xyz.r(1) = TrackBefMove.pos.xyz.r(1)
-     *                      + TrackBefMove.vec.w.r(1) * leng
+c     *                      + TrackBefMove.vec.w.r(1) * leng
+     *                      + dir.r(1) * leng
        MovedTrack.pos.xyz.r(2) = TrackBefMove.pos.xyz.r(2)
-     *                      + TrackBefMove.vec.w.r(2) * leng
+c     *                      + TrackBefMove.vec.w.r(2) * leng
+     *                      + dir.r(2) * leng
        MovedTrack.pos.xyz.r(3) = TrackBefMove.pos.xyz.r(3)
-     *                      + TrackBefMove.vec.w.r(3) * leng
+c     *                      + TrackBefMove.vec.w.r(3) * leng
+     *                      + dir.r(3) * leng
        call csetPos(MovedTrack.pos) ! set height, radial length, thickness
 c            get new cos
        MovedTrack.vec.coszenith = cnewcos(MovedTrack.pos.radiallen,
@@ -136,7 +141,7 @@ c           see if <=mass
      *          TrackBefMove.vec.coszenith, thick, leng, thick, jcut)
 
 c               reset position information in MovedTrack
-              call cmoveStreight(leng)
+              call cmoveStreight(leng, TrackBefMove.vec.w)
               MoveStat = Truncated
            endif
 c                reset 3 momenta px, py, pz 
@@ -206,7 +211,8 @@ c                compute excess path length to be added to streight path
 c      *************************
        subroutine cmagDef
        implicit none
-
+#include  "Zobs.h"
+#include  "Zobsp.h"
 #include  "Ztrack.h"
 #include  "Ztrackp.h"
 #include  "Ztrackv.h"
@@ -216,28 +222,54 @@ c
 
 c              by  Geomag  (dr and ddirec)
        record /coord/ dispmr, dispmd
-       real*8  leng,  norm
+       real*8  leng
+ccc    *   ,norm
        integer i
 
+c^^^^^^^^^   for trial to use B at middle of the path
+c       integer icon
+c       record /coord/  middle
+c^^^^^^^^^
+
        leng = IntInfArray(ProcessNo).length
+
+c^^^^^^^^^
+c       middle.x=TrackBefMove.pos.xyz.x + 
+c     *          leng/2 * TrackBefMove.vec.w.x
+c       middle.y=TrackBefMove.pos.xyz.y +
+c     *          leng/2 * TrackBefMove.vec.w.y
+c       middle.z=TrackBefMove.pos.xyz.z + 
+c     *          leng/2 * TrackBefMove.vec.w.z
+c       middle.sys='xyz'
+
+c       call cgeomag(YearOfGeomag,  middle,
+c     *                Mag, icon)
+c       call ctransMagTo('xyz', middle,
+c     *        Mag, Mag)
+c^^^^^^^^^^^
+
        call cmagneticDef(TrackBefMove, Mag, leng,
      *     dispmr, dispmd)
 
        do i = 1,  3
 c               displacement
-          MovedTrack.pos.xyz.r(i) = MovedTrack.pos.xyz.r(i) +
+c            for obso routine
+c          MovedTrack.pos.xyz.r(i) = MovedTrack.pos.xyz.r(i) +
+c     *         dispmr.r(i)
+          MovedTrack.pos.xyz.r(i) = TrackBefMove.pos.xyz.r(i) +
      *         dispmr.r(i)
-c               direction
-          MovedTrack.vec.w.r(i) = MovedTrack.vec.w.r(i) +
-     *         dispmd.r(i)
-       enddo
-c           normalize
-       norm = sqrt(MovedTrack.vec.w.r(1)**2 +
-     *            MovedTrack.vec.w.r(2)**2 +
-     *            MovedTrack.vec.w.r(3)**2)
-       do i = 1, 3
-           MovedTrack.vec.w.r(i) = MovedTrack.vec.w.r(i)/norm
+c               direction; for obso
+cccc          MovedTrack.vec.w.r(i) = MovedTrack.vec.w.r(i) +
+cccc     *         dispmd.r(i)
+          MovedTrack.vec.w = dispmd
        enddo
+cc           normalize
+cc       norm = sqrt(MovedTrack.vec.w.r(1)**2 +
+cc     *            MovedTrack.vec.w.r(2)**2 +
+cc     *            MovedTrack.vec.w.r(3)**2)
+cc       do i = 1, 3
+cc           MovedTrack.vec.w.r(i) = MovedTrack.vec.w.r(i)/norm
+cc       enddo
        end
 c      ==============================================================
        subroutine celecDef(dispx, dispy)
