@@ -26,6 +26,18 @@ c
 		target(i:i)=Cstring(i)
 	enddo
        end
+#elif defined DECALPHA
+      subroutine cgetLoginN(userid)
+      implicit none
+      character*8 userid
+      character*16 user
+      character*1 NULL
+      integer leng, kgetenv
+      
+      NULL = char(0)
+      leng = kgetenv("USER"//NULL, user)
+      userid = user(1:leng)
+      end
 #else
 c     **************************** sun4/hp ***************************
 c       !!!!  for HP you need +U77 compiler option
