@@ -2,7 +2,6 @@ C            cifDead;  see if too low energy
 c
         subroutine cifDead(a, icon)
         implicit none
-
 #include  "Ztrack.h"
 #include  "Zcode.h"
 #include  "Ztrackv.h"
@@ -11,8 +10,9 @@ c
 #include  "Zobsp.h"
 c
       record /track/ a   ! input.  a track to be examined
-      integer icon !   output.  0 => alive  else dead
-
+      integer icon !   output.  0 ==> alive  
+                   !            1 ==> energy < threshold
+                   !            2 ==> path exceeded the limit 
       real*8  ke
       integer jold
 
@@ -62,7 +62,7 @@ c               can decay
            endif
         endif
         if(PathLimit .lt. a.t) then
-           icon = 1
+           icon = 2
         endif
         if( a.where .gt. EndLevel2) then
            call rndsw(jold, 2)
