@@ -1,4 +1,4 @@
-       subroutine cifXObsSite
+       subroutine cifXObsSite(nextwhere)
 c         see if MovedTrack crosses an observation depth.
 c         or go out of boundry.
 c        if to be observed, reset MovedTrack information
@@ -12,13 +12,16 @@ c        if to be observed, reset MovedTrack information
 #include  "Zobsv.h"
 #include  "Zearth.h"
 c     
+      integer nextwhere  ! output next where. to be used when ptcl
+                         !  crosses an  observation  level. spherical case.
       integer loc
       real*8  leng, dedt, cosfromaxis
       real*8  clen2thick
 
-      logical cross
+      logical cross, ok
+      integer  mustbe
       record /coord/ xyz1, xyz2, dircos, diffvec
-      real*8 dummylen, r0, r0r, d
+      real*8 dummylen, r0, r0r, d, obsrl, height
 c
 c     ObsPlane == horizontal:
 c                 Although the observation plane is horizontal (tangential at a
@@ -36,14 +39,6 @@ c
 c                  
 c
       cross = .false.
-c       if go out of the boundary,   discard it
-      if(MovedTrack.pos.height .ge. BorderHeightH) then
-         MoveStat = BorderH
-         return                 !  ***************
-      endif
-      if(MovedTrack.pos.height .le. BorderHeightL) then
-         MoveStat = BorderL
-      endif
 
       if(ObsPlane .ne. notUsed ) then
          loc = MovedTrack.where 
@@ -136,45 +131,108 @@ c           convert coord into 1ry system
                endif
             endif
          elseif(ObsPlane .eq. spherical) then
-            if(TrackBefMove.vec.coszenith .gt. 0.) then
-               if( MovedTrack.pos.radiallen .le. 
-     *              ObsSites(loc).pos.radiallen  .and.
-     *             TrackBefMove.pos.radiallen .gt. 
-     *                        ObsSites(loc).pos.radiallen ) then
-                  cross = .true.
-               elseif(loc .lt. NoOfSites) then 
-                  if( TrackBefMove.pos.radiallen .gt.
-     *                    ObsSites(loc+1).pos.radiallen .and.
-     *                MovedTrack.pos.radiallen .le.
-     *                    ObsSites(loc+1).pos.radiallen) then
-                     cross =.true.
-                     loc = loc + 1
-                     MovedTrack.where = loc 
-                     TrackBefMove.where = loc
-                  endif
+            if( TrackBefMove.pos.height .ge. 
+     *            ObsSites(loc).pos.height  .and.
+     *            MovedTrack.pos.height .lt. 
+     *            ObsSites(loc).pos.height ) then
+               cross = .true.
+               height = ObsSites(loc).pos.height
+               mustbe = -1
+c               MovedTrack.where =loc
+               if(TrackBefMove.vec.coszenith .ge. 0.) then
+                  nextwhere = min(loc+1, NoOfSites)
+               else
+                  nextwhere = max(loc-1, 1)
                endif
-            else
-               if(TrackBefMove.pos.radiallen .lt.  
-     *                ObsSites(loc).pos.radiallen  .and.
-     *            MovedTrack.pos.radiallen .ge. 
-     *                ObsSites(loc).pos.radiallen) then
+            elseif( TrackBefMove.pos.height .le. 
+     *            ObsSites(loc).pos.height  .and.
+     *            MovedTrack.pos.height .gt. 
+     *            ObsSites(loc).pos.height ) then
+               cross = .true.
+               height = ObsSites(loc).pos.height
+               mustbe = 1
+c               MovedTrack.where = loc
+               if(TrackBefMove.vec.coszenith .le. 0.) then
+                  nextwhere = max(loc-1, 1)
+               else
+                  nextwhere = min(loc+1, NoOfSites)
+               endif
+            elseif(loc .gt. 1) then
+               if( TrackBefMove.pos.height .le.
+     *                    ObsSites(loc-1).pos.height .and.
+     *                MovedTrack.pos.height .gt.
+     *                    ObsSites(loc-1).pos.height ) then
+                  cross =.true.
+                  height = ObsSites(loc-1).pos.height
+                  mustbe = 1 
+                  loc = loc-1
+                  MovedTrack.where = loc
+c                  TrackBefMove.where = loc
+                  nextwhere = max(loc-1,1)
+               endif
+            elseif(loc .lt. NoOfSites) then
+               if(TrackBefMove.pos.height .ge.  
+     *                ObsSites(loc+1).pos.height .and.
+     *            MovedTrack.pos.height .lt. 
+     *                ObsSites(loc+1).pos.height ) then
                   cross = .true.
-               elseif(loc .gt. 1) then
-                  if(TrackBefMove.pos.radiallen .lt. 
-     *                   ObsSites(loc-1).pos.radiallen .and.
-     *               MovedTrack.pos.radiallen .ge. 
-     *                   ObsSites(loc-1).pos.radiallen) then
-                     cross = .true.
-                     loc = loc -1
-                     MovedTrack.where = loc
-                     TrackBefMove.where = loc
-                  endif
+                  height = ObsSites(loc+1).pos.height 
+                  mustbe = -1
+                  loc = loc +1
+                  MovedTrack.where = loc 
+                  nextwhere = min(loc+1, NoOfSites)
+               endif
+            endif
+c            if(.not. cross .and.  loc .eq. NoOfSites) then
+            if(.not. cross ) then
+               if(TrackBefMove.pos.height .ge. BorderHeightL .and.
+     *               MovedTrack.pos.height .lt. BorderHeightL)  then
+c                   don't set cross
+                  MovedTrack.where = -1 
+                  MoveStat = BorderL
+               endif
+            endif
+c            if(.not. cross .and. MoveStat .ne.
+c     *                   BorderL .and. loc .eq. 1) then
+            if(.not. cross .and. MoveStat .ne. BorderL ) then
+               if(TrackBefMove.pos.height .le. BorderHeightH .and.
+     *            MovedTrack.pos.height .gt. BorderHeightH ) then
+c                    don't set cross
+                  MovedTrack.where = 0 
+                  MoveStat = BorderH
+               endif
+            endif
+            if(.not. Cross .and. (MoveStat .ne. BorderL) .and.
+     *          (MoveStat .ne. BorderH) ) then
+c               this will happen, for example, the starting
+c               depth is lower than BorderL(by epsilon due to
+c               numerical innaccuracy). assume the ptcl crossed
+c               the border
+               if( TrackBefMove.pos.height .ge. BorderHeightH .and. 
+     *             MovedTrack.pos.height .ge. BorderHeightH )  then
+                  MoveStat = BorderH
+                  MovedTrack.where = 0
+               elseif( TrackBefMove.pos.height .le. BorderHeightL
+     *                 .and. 
+     *              MovedTrack.pos.height .le. BorderHeightL) then
+                  MoveStat = BorderL
+                  MovedTrack.where = -1
                endif
             endif
          endif
-
+c//////////
+c         write(*,*) 'corss=',cross, ' Move=',MoveStat,
+c     *          ' MovedTrack.pos.height=',MovedTrack.pos.height,
+c     *          ' TrackBefMove.pos.height', TrackBefMove.pos.height
+c/////////
          if(cross) then
             MoveStat = ToBeObserved
+c            write(*,*)' loc=',loc, ' h1=', TrackBefMove.pos.height,
+c     *    ' obsh=',  ObsSites(loc).pos.height, 
+c     *    ' h2=',  MovedTrack.pos.height
+         endif
+         if(cross .or. MoveStat .eq. BorderH .or. 
+     *       MoveStat .eq. BorderL) then
 c             get length, leng, from the starting point to
 c             the observation depth
             call cdiffvec(TrackBefMove.pos.xyz, 
@@ -226,12 +284,19 @@ c                                   r0                 r
                call cscalerProd(TrackBefMove.pos.xyz, diffvec,
      *               r0r)
                r0 = TrackBefMove.pos.radiallen
-               if(r0 .le. ObsSites(loc).pos.radiallen) then
+               if( MovedTrack.where .gt. 0) then
+                  obsrl =  ObsSites(loc).pos.radiallen
+               elseif(MovedTrack.where .eq. 0) then
+                  obsrl = BorderHeightH + Eradius
+               else
+                  obsrl = BorderHeightL + Eradius
+               endif
+               if(r0 .le. obsrl ) then
                   leng = -r0r + sqrt(r0r**2 -
-     *             (r0**2 - ObsSites(loc).pos.radiallen**2))
+     *             (r0**2 - obsrl**2))
                else
                   leng = -r0r - sqrt(r0r**2 -
-     *              (r0**2-ObsSites(loc).pos.radiallen**2))
+     *              (r0**2-obsrl**2))
                endif
                leng = leng + 0.01d0
             endif
@@ -250,10 +315,28 @@ c                          use straight line approximation here.
 c             reset MovedTrack
 c            MovedTrack = TrackBefMove   !  not needed
             call cmoveStreight(leng, diffvec)
-               
+            if(ObsPlane .eq. spherical) then
+c                  confirm that ptcl crosses the plane
+               if(MoveStat .ne. BorderH .and.
+     *            MoveStat .ne. BorderL) then
+                  ok = .false.
+                  do while (.not. ok)
+                     if(mustbe .eq. 1) then
+                        ok = MovedTrack.pos.height .gt. height
+                     else
+                        ok = MovedTrack.pos.height .lt. height
+                     endif
+                     if(.not. ok) then
+                        leng = leng*1.0001d0
+                        call cmoveStreight(leng, diffvec)
+                     endif
+                  enddo
+               endif
+            endif
+
             if(MovedTrack.p.charge .ne. 0) then
                call cqElossRate(dedt) ! get de/dx computed previously
-               MovedTrack.p.fm.p(4) =max( MovedTrack.p.fm.p(4) -
+               MovedTrack.p.fm.p(4) =max( TrackBefMove.p.fm.p(4) -
      *          dedt * IntInfArray(ProcessNo).thickness,
      *          MovedTrack.p.mass)   ! max is for safety. we neg. 
                                      ! syncrotron loss
@@ -268,13 +351,12 @@ c            endif
          endif
       endif
       if(.not. cross .or. ObsPlane .eq. notUsed) then
-c            see if go out of outer boundery.
+c        
          if(BackAngLimit .gt. -1.0) then
             call cscalerProd(MovedTrack.vec.w, DcAtObsXyz, cosfromaxis)
             if(cosfromaxis .lt. BackAngLimit) then
 c                discard it
-               MoveStat = BorderH
-               return           !  ***************
+               MoveStat = AngleLimit
             endif
          endif
       endif
