@@ -56,10 +56,8 @@ c             interface with user hook  ***********************
              call chookNEPInt(neverNEP)
              never = neverNEP
           endif
-
 c             *******************************
       endif
-
       if( never .eq. 1 .or. never .eq. 0) then
          if(OneDim .eq. 0) then
 c            3 dimensional
@@ -103,6 +101,7 @@ c          3 dimensional case.
 
       subroutine  cmovePtcl3(iTrack, pw, n)
       implicit none
+#include  "Zcode.h"
 #include  "Ztrackp.h"
 #include  "Ztrack.h"
 #include  "Ztrackv.h"
@@ -120,7 +119,6 @@ c
       aTrack = iTrack
 
       do i =  n, 1, -1          ! move leading ptcl first
-
 #ifdef SUBSTREC
          aTrack.p = pw(i)
 #else
@@ -140,9 +138,96 @@ c               reset direction cos and related stuffs
          else
             icon = 0
          endif
-         if(icon .eq.0) call cpush(aTrack)
-      enddo
+         if(icon .eq.0) then
+#if LABELING == 1
+c
+c              whennever an interaction occur,  update labelcounter
+c             if info>0, clear the timer and infor counters.
+c
+            if(aTrack.info .gt. 0) then
+               aTrack.info = 0
+               aTrack.t = 0.
+            endif
+            Labelcounter = Labelcounter + 1
+            aTrack.label = Labelcounter
+#elif LABELING == 2
+c            the above simple counter may be replaced by the
+c            next sophisticated one. 
+c            the same one is in the 1dim mode move routine below.
+c
+            call ctrickycount(iTrack, aTrack, pw, i)
+c
+#endif
+            call cpush(aTrack)
+         endif
+      enddo 
       end
+#if LABELING == 2
+c       ************************************
+c             This routine updates label counters.
+c          but for the survival particle form brems and knock-on
+c          the label counter is not updated. 
+c          For those ptcl with info > 0, timer and info counters
+c          are cleared. 
+      subroutine ctrickycount(iTrack, aTrack, pw, i)
+      implicit none
+#include  "Zcode.h"
+#include  "Ztrackp.h"
+#include  "Ztrack.h"
+#include  "Ztrackv.h"
+c
+      integer n
+      record /track/ iTrack  ! input. incident ptcl.
+      record /track/ aTrack  ! input/output. i-th  secondary track 
+      record /ptcl/ pw(*)    ! input.  secondary pool
+      integer i              ! input.  i-th secondary (index)
+
+      logical reset
+
+      reset = .true.
+
+      if(IntInfArray(ProcessNo).process .eq. 'brems' ) then
+c            for brem, electron has the same label as the incident
+         if(aTrack.p.code .eq. kelec ) then
+            aTrack.label = iTrack.label
+            reset = .false.
+         endif
+      elseif(IntInfArray(ProcessNo).process .eq. 'knock' .or.
+     *         IntInfArray(ProcessNo).process .eq. 'mscat' .or.
+     *         IntInfArray(ProcessNo).process .eq. 'bscat' ) then
+c                for the knock-on, survival particle has the same
+c                label.
+         if(iTrack.p.code .ne. kelec  .or.
+     *        iTrack.p.charge .ne. -1) then
+c                     knockon by p,mu,pi..e+ (not by e-)
+            if(aTrack.p.code .ne. kelec) then
+c                       survival one has the same label
+               aTrack.label = iTrack.label
+               reset = .false.
+            endif
+         else
+c                  electron; make the higher one has the same label
+            if(pw(1).fm.p(4) .gt. pw(2).fm.p(4)) then
+               if(i .eq. 1) then
+                  aTrack.label = iTrack.label
+                  reset = .false.
+               endif
+            else
+               if(i .eq. 2) then
+                  aTrack.label = iTrack.label
+                  reset = .false.
+               endif
+            endif
+         endif                     
+      endif
+      if(reset) then
+         Labelcounter = Labelcounter +1
+         aTrack.label = Labelcounter
+         aTrack.t = 0.          ! timer reset
+         aTrack.info = 0        ! cross counter reset
+      endif
+      end
+#endif
 c     ************************************
 c          move partcles in a given array to stack
 c          1 dimensional case.
@@ -150,7 +235,7 @@ c          1 dimensional case.
       subroutine  cmovePtcl1(iTrack, pw, n)
       implicit none
 
-
+#include  "Zcode.h"
 #include  "Ztrack.h"
 #include  "Ztrackp.h"
 #include  "Ztrackv.h"
@@ -202,7 +287,26 @@ c            call cresetMom(aTrack)  which is
             else
                icon = 0
             endif
-            if(icon .eq. 0) call cpush(aTrack)
+            if(icon .eq. 0) then
+#if LABELING == 1
+c                  whennever secondary particles are generated,
+c               each of them get an updated label cocunter
+c               if the particle has crossed the highest level
+c               (info > 0),  timer and info counter is cleared
+c
+               if( aTrack.info .gt. 0) then
+                  aTrack.info = 0
+                  aTrack.t = 0.
+               endif
+               Labelcounter = Labelcounter + 1
+               aTrack.label = Labelcounter
+#elif LABELING == 2
+c                   this may be used if a tricky count is needed
+c               in stead of above  counting
+               call ctrickycount(iTrack, aTrack, pw, i)
+#endif
+               call cpush(aTrack)
+            endif
          endif
       enddo
       end
@@ -266,3 +370,5 @@ c     ***************************
       end
 
 
+
+
