@@ -37,6 +37,9 @@ c            (in this case Za1ry == 'is' guaranteed).
 
          call csPrimAng(angle)
          call cmkInc(incident, angle)
+#if LABELING > 0
+         incident.info = 0
+#endif
 c           
          if(ObsPlane .eq. spherical) then
 c              reset position and angle.
@@ -53,6 +56,8 @@ c              reset position and angle.
                              ! which is not rejected. 
          endif
       enddo
+
+      
  10   continue
       end
 #ifdef NEXT486
@@ -260,7 +265,7 @@ c         primary is going upward even if Reverse = 0.
       if(( Reverse .eq. 0 .and. zenithcos .lt. 0. 
      *  .and.  HeightOfInj .gt. ObsSites(NoOfSites).pos.height)
      *  .or. ( Reverse .ne. 0 .and. zenithcos .gt. 0.)) then
-         if(Obsplane .ne. spherical) then
+         if(ObsPlane .ne. spherical) then
 c            distance to the conjugate point
             to100km = to100km -
      *       2*(ObsSites(NoOfSites).pos.radiallen + OffsetHeight)*
