@@ -21,26 +21,11 @@ c      ***  do until no more ptcl in stack
       do while (.true.)
 c           get one particle from stack
          call cpop(TrackBefMove, moreptcl)
-
          if(moreptcl .eq. 0) goto 100   !  exit
          call rndsw(jold, 1)            !  specify 1st generator.
          call cifDead(TrackBefMove, icon)  ! generator may be switched to 2
 c            AS generation. 
 
-c         if(icon .eq. 0) then
-c            thin sampling
-c            if(TrackBefMove.p.fm.p(4) .lt. Ethin) then
-c                do thin sampling
-c               iwgt = TrackBefMove.p.fm.p(4)/Ethin ! inverse weight
-c               call rndc(u)
-c               if(u .gt. iwgt) then
-c                  icon =2       !  discard this ptcl
-c               else
-c                  TrackBefMove.wgt = TrackBefMove.wgt /iwgt
-c               endif
-c            endif    
-c         endif
-
          if(ObserveAS .and. icon .ne. 2) then
             if(TrackBefMove.p.code .eq. kelec) then
                if(TrackBefMove.asflag .eq. 0) then
@@ -64,6 +49,16 @@ c         endif
 c        ----------------------------
          if(icon .eq. 0) then
             call cTracking
+         else
+            MoveStat = Dead
+#if DETAILED_TRACKING >=1
+            MovedTrack = TrackBefMove
+            if(icon .eq. 1) then
+               call cobservation(5)
+            else
+               call cobservation(6)
+            endif
+#endif
          endif
       enddo
  100  continue
@@ -82,10 +77,11 @@ c -------------------------------------tracking a ptcl
 #include  "Zcode.h"
 
 c                next is for DEBUG
-#include  "Zcondc.h"
+
 
       real*8 cosangle
       character*70 msg
+      integer nextwhere
 cc      integer icon
 
 
@@ -102,8 +98,6 @@ c           path in the above process may be again truncated to
 c           a shorter one. This happens, for example, if ptcl energy
 c           is low.  Path inf is reset in IntInfArray.
          call ctruncPath
-
-
 #if DEBUG == 1
          call checkstat('after ctruncPath')
 #endif
@@ -111,21 +105,24 @@ c           is low.  Path inf is reset in IntInfArray.
 c           compute path end inf. including scattering and
 c           mag. deflection.  Path end inf is set in MovedTrack
          call ccompPathEnd
-
 #if DEBUG == 1
          call checkstat('after ccompPathEnd')
 #endif
 c           see if MovedTrack crosses an observation site.
 c         (or reaches BorderHeightL)  If so, reset MovedTrack. 
-            call cifXObsSite
-
+            call cifXObsSite(nextwhere)
 #if DEBUG == 4
             call checkstat('after cifXObsSite')
 #endif
-
          if(Trace .ne. 0) then
             call cputTrInfo     ! put trace info.
          endif
+#if DETAILED_TRACKING >= 3
+c            the user may kill the ptcl
+         call cobservation(8)
+#endif
+
+
 c     
          if(MoveStat .eq. Truncated) then
 c           see if truncation is really ok.  for energy = 0, 
@@ -143,6 +140,7 @@ c           it should be decayed.
                endif
             endif
          endif 
+
          if(MoveStat .eq. Truncated) then
 c              stack current ptcl
               call cpush(MovedTrack)
@@ -157,6 +155,9 @@ c                  the first collision point. or ptcl goes streight
                   call csetMinTime(MovedTrack)
                endif
             endif
+#if DETAILED_TRACKING >= 1 
+            call cobservation(4)
+#endif
             call cinteraction
          elseif(MoveStat .eq. ToBeObserved) then
             if(MovedTrack.p.fm.p(4)-MovedTrack.p.mass 
@@ -171,53 +172,64 @@ c               update observation place
             elseif(abs(ObsPlane) .eq. perpendicular) then
                call cscalerProd(MovedTrack.vec.w, DcAtObsXyz,
      *               cosangle)
-            elseif(ObsPlane .eq. spherical) then
-               call cscalerProd(MovedTrack.vec.w, MovedTrack.pos,
-     *               cosangle)
-               cosangle = - cosangle
-            else
-               call 
-     *         cerrorMsg('should not come here in tracking', 0)
-            endif
-
-            if(cosangle .lt. 0.) then
-               MovedTrack.where = MovedTrack.where - 1
-            else
-               MovedTrack.where = MovedTrack.where + 1
             endif
+c            elseif(ObsPlane .eq. spherical) then
+c               call cscalerProd(MovedTrack.vec.w, MovedTrack.pos,
+c     *               cosangle)
+c               cosangle = - cosangle
+c            else
+c               call 
+c     *         cerrorMsg('should not come here in tracking', 0)
+c            endif
+            if(ObsPlane .ne. spherical) then
+               if(cosangle .lt. 0.) then
+                  MovedTrack.where = MovedTrack.where - 1
+               else
+                  MovedTrack.where = MovedTrack.where + 1
+               endif
 c                  if more obs-site, stack current ptcl.
-            if(.not. Upgoing) then  ! incident is downgoing
-               if(MovedTrack.where .gt. EndLevel) then
-                  ! no need to stack; discard ptcl.
-               else
-                  MovedTrack.where =max( MovedTrack.where, 1)
-                  call cpush(MovedTrack)
-               endif
-            else    !incident is  Upgoing
-               if(MovedTrack.where .lt.1 ) then
-                   ! no need to stack
-               else
-                  MovedTrack.where =
+               if(.not. Upgoing) then ! incident is downgoing
+                  if(MovedTrack.where .gt. EndLevel) then
+                                ! no need to stack; discard ptcl.
+                  else
+                     MovedTrack.where =max( MovedTrack.where, 1)
+                     call cpush(MovedTrack)
+                  endif
+               else             !incident is  Upgoing
+                  if(MovedTrack.where .lt.1 ) then
+                                ! no need to stack
+                  else
+                     MovedTrack.where =
      *                         min(MovedTrack.where, EndLevel)
-                  call cpush(MovedTrack)
+                     call cpush(MovedTrack)
+                  endif
                endif
+            else
+               MovedTrack.where = nextwhere
+               call cpush(MovedTrack)
             endif
          elseif(MoveStat .eq. BorderL) then
             call cobservation(3)
          elseif(MoveStat .eq. BorderH) then
             call cobservation(1)
          elseif(MoveStat .eq. Dead) then   
-c              nothing to do.
+c            actually this will not happen. branch is
+c           made before this
+#if DETAILED_TRACKING >= 1
+            call cobservation(5)
+#endif
+         elseif(MoveStat .eq. AngleLimit) then
+#if DETAILED_TRACKING >= 2
+            call cobservation(7)
+#endif
          else
             write(msg, *) ' movestat=',MoveStat,' invalid'
            call cerrorMsg(msg, 0)
          endif   
-
       end
 #if DEBUG > 0
       subroutine checkstat(str)
       implicit none
-
 #include  "Ztrack.h"
 #include  "Ztrackv.h"
 #include  "Ztrackp.h"
@@ -291,8 +303,10 @@ c             distant; change of B is < 1 %
      *        .gt. distant**2) then
 c              if more than MagChgDist m from previous mag set.,
 c               reset mag field.
+c
             call cgeomag(YearOfGeomag,  TrackBefMove.pos.xyz, 
      *                Mag, icon)
+
             call ctransMagTo('xyz', TrackBefMove.pos.xyz, 
      *        Mag, Mag)
             x = TrackBefMove.pos.xyz.r(1)
