@@ -13,17 +13,19 @@ c              test primary sampling routines
 
       real*8 kin
       integer label, icon, i
-
+      real*4 mom, wgt
+      integer fin
 
       call creadParam(5)
       call cbeginRun
       call cprintPrim(ErrorOut)
       write(ErrorOut, 
      *   '("#seq. No.|   Orig Erg | Code # | Total E(GeV)")')
-      do i = 1, DestEventNo(1)
+      do i = 1, abs(DestEventNo(1))
          icon = 1
          do while (icon .ne. 0)
-            call csampPrimary(incident.p)
+            call csampPrimary(incident.p, fin)
+            if(fin .ne. 0) goto 10 
             call csPrimAng(angle)
             call cmkInc(incident, angle)
             if(CutOffFile .ne. ' ') then
@@ -32,11 +34,15 @@ c              test primary sampling routines
               icon =0
             endif
          enddo
+ 10      continue
          call cqPrimE(kin)
          call cqPrimLabel(label)
          p = incident.p
+         mom =  sngl(sqrt(p.fm.p(4)**2 - p.mass**2))
+         wgt =0.2323*mom**1.7       
          write(*, *) label, sngl(kin),
-     *      p.code,  sngl(p.fm.p(4)), sngl(log10(kin)), sngl(1/kin)
+     *      p.code,  sngl(p.fm.p(4)), sngl(log10(kin)),
+     *      sngl(0.2323/kin),  log10(mom), wgt
       enddo
       end
       subroutine chookTrace
